#Copyright (C) 2005 by Dr. Dieter Maurer <dieter@handshake.de> -- see LICENSE.txt
#       $Id: path.py,v 1.2 2005/05/27 12:09:14 dieter Exp $
'''Variants for ``os.path`` functions handling both filesystem and ``pypackage:`` references.'''

from os import path as _path

from util import getBuiltin
from ReuseUtils import rebindFunction
import pypackage

_os_rebinds = ('getsize getmtime getatime getctime '
               'islink exists isdir isfile ' 
               'walk '
               ).split()

_newly_implemented = 'isabs join normpath abspath'.split()

_mdict = globals()

for _f in _os_rebinds + _newly_implemented:
    _mdict['_builtin__' + _f] = getBuiltin(_path, _f)

for _f in _os_rebinds:
    _mdict[_f] = rebindFunction(_mdict['_builtin__' + _f], os=pypackage)


def isabs(file):
    if file.startswith('pypackage:'): return True
    return _builtin__isabs(file)


def normpath(file):
    if file.startswith('pypackage:'):
        return pypackage.join(*pypackage.split(file))
    return _builtin__normpath(file)

def abspath(path):
    if path.startswith('pypackage:'): return path
    return _builtin__abspath(path)

def join(base, *p):
    segs = (base,) + p
    # find the last 'pypackage' component.
    # We know it eradicates anything before it
    i = len(segs)
    while i:
        i -= 1
        seg = segs[i]
        if seg.startswith('pypackage:'): break
    else: i = None
    if i is None: return _builtin__join(*segs)
    segs = segs[i:]
    # find the first absolute component
    # it eradicates anything before it
    for i in range(1, len(segs)):
        seg = segs[i]
        if _builtin__isabs(seg):
            return _builtin__join(*segs[i:])
    # the result is a 'pypackage' URL
    return normpath('/'.join(segs))
